#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
@author: Olegs Nikisins
"""
#==============================================================================
# Import here:

from torch import nn

#==============================================================================
# Define the network:

class ConvAutoencoder(nn.Module):
    """
    A class defining a simple convolutional autoencoder.

    Attributes
    ----------
    return_latent_embedding : bool
        If set to ``True`` forward() method returns a latent
        emebedding (encoder output), otherwise a reconstructed
        image is returned. Default: ``False``
    """

    def __init__(self, return_latent_embedding = False):
        super(ConvAutoencoder, self).__init__()

        self.return_latent_embedding = return_latent_embedding

        self.encoder = nn.Sequential(nn.Conv2d(3, 16, 5, padding=2),
                                     nn.ReLU(True),
                                     nn.MaxPool2d(2),
                                     nn.Conv2d(16, 16, 5, padding=2),
                                     nn.ReLU(True),
                                     nn.MaxPool2d(2),
                                     nn.Conv2d(16, 16, 3, padding=2),
                                     nn.ReLU(True),
                                     nn.MaxPool2d(2),
                                     nn.Conv2d(16, 16, 3, padding=2),
                                     nn.ReLU(True),
                                     nn.MaxPool2d(2))

        self.decoder = nn.Sequential(nn.ConvTranspose2d(16, 16, 3, stride=2, padding=1),
                                     nn.ReLU(True),
                                     nn.ConvTranspose2d(16, 16, 3, stride=2, padding=1),
                                     nn.ReLU(True),
                                     nn.ConvTranspose2d(16, 16, 5, stride=2, padding=2),
                                     nn.ReLU(True),
                                     nn.ConvTranspose2d(16, 3, 5, stride=2, padding=2),
                                     nn.ReLU(True),
                                     nn.ConvTranspose2d(3, 3, 2, stride=1, padding=1),
                                     nn.Tanh())

    def forward(self, x):
        """
        The forward method.
        """
        x = self.encoder(x)

        if self.return_latent_embedding:
            return x

        x = self.decoder(x)

        return x

