
from torchvision import transforms

from bob.learn.pytorch.architectures import MCCNN

from bob.learn.pytorch.datasets import DataFolder

from bob.pad.face.database import BatlPadDatabase

from bob.learn.pytorch.datasets import ChannelSelect


#==============================================================================
# Load the dataset

""" The steps are as follows

1. Initialize a databae instance, with the protocol, groups and number of frames 
	(currently for the ones in 'bob.pad.face', and point 'data_folder_train' to the preprocessed directory )	
	Note: Here we assume that we have already preprocessed the with `spoof.py` script and dumped it to location 
	pointed to by 'data_folder_train'.

2. Specify the transform to be used on the images. It can be instances of `torchvision.transforms.Compose` or custom functions.

3. Initialize the `data_folder` class with the database instance and all other parameters. This dataset instance is used in
 the trainer class

4. Initialize the network architecture with required arguments.

5. Define the parameters for the trainer. 

"""

#==============================================================================
# Initialize the bob database instance 

data_folder_train='/idiap/temp/ageorge/WMCA/preprocessed/'

frames=50

extension='.h5'

train_groups=['train'] # only 'train' group is used for training the network

protocols="grandtest-color*depth*infrared*thermal-{}".format(frames) # makeup is excluded anyway here

exlude_attacks_list=["makeup"]

bob_hldi_instance_train = BatlPadDatabase(
    protocol=protocols,
    original_directory=data_folder_train,
    original_extension=extension,
    landmark_detect_method="mtcnn",  # detect annotations using mtcnn
    exclude_attacks_list=exlude_attacks_list,
    exclude_pai_all_sets=True,  # exclude makeup from all the sets, which is the default behavior for grandtest protocol
    append_color_face_roi_annot=False) 

#==============================================================================
# Initialize the torch dataset, subselect channels from the pretrained files if needed.

SELECTED_CHANNELS = [0,1,2,3] # selects  color, depth, infrared and thermal

img_transform_train = transforms.Compose([ChannelSelect(selected_channels = SELECTED_CHANNELS),transforms.ToPILImage(),transforms.RandomHorizontalFlip(),transforms.ToTensor()])# Add p=0.5 later

dataset = DataFolder(data_folder=data_folder_train,
					 transform=img_transform_train,
					 extension='.hdf5',
					 bob_hldi_instance=bob_hldi_instance_train,
					 groups=train_groups,
					 protocol=protocols,
					 purposes=['real', 'attack'],
					 allow_missing_files=True)

#==============================================================================
# Load the architecture

NUM_CHANNELS = 4

assert(len(SELECTED_CHANNELS)==NUM_CHANNELS)

network=MCCNN(num_channels = NUM_CHANNELS)

#==============================================================================
# Specify other training parameters

batch_size = 64
num_workers = 4
epochs=25
learning_rate=0.0001
seed = 3
output_dir = 'training_mccn'
use_gpu = False
adapted_layers = 'conv1-group1-block1-ffc'
adapt_reference_channel = False
verbose = 2


#==============================================================================
"""
Note: Running in GPU

jman submit --queue gpu \
--name mccnn \
--log-dir /idiap/user/ageorge/WORK/COMMON_ENV_PAD_BATL_DB/training_mccn/logs/ \
--environment="PYTHONUNBUFFERED=1" -- \
./bin/train_mccnn.py \
/idiap/user/ageorge/WORK/COMMON_ENV_PAD_BATL_DB/src/bob.learn.pytorch/bob/learn/pytorch/config/mccnn/wmca_mccnn.py --use-gpu -vvv

Note: Running in cpu

./bin/train_mccnn.py \
/idiap/user/ageorge/WORK/COMMON_ENV_PAD_BATL_DB/src/bob.learn.pytorch/bob/learn/pytorch/config/mccnn/wmca_mccnn.py -vvv

"""

